#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#Datos: Todas las clases relacionadas con los datos, y sus correspondientes metodos

#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import variables
import types
from qt import QString #TODO: Poner solo en el lugar que sea necesario


def ParseCondicion(datos,ristra):
	"""Funcion que devuelve un objeto condicion a partir de una ristra que ha pasado el usuario"""
	import re
	esseleccion=re.compile(".*=.*")
	if esseleccion.match(ristra):
		lista=ristra.split("=")
		assert(len(lista)==2)
		var=datos.getVar(lista[0])
		elem=var.nuevoitem(lista[1])
		return Seleccion(var,elem)
	

class Condicion:
	"""Esta clase expresara una condicion parecida a las de tipo SQL"""
	pass

class Seleccion(Condicion):
	""" Esto es una condicion de filtrado de seleccion es equivalente en SQL 'where variable=valor'"""
	def __init__(self,variable,valorobjetivo):
		assert(valorobjetivo.tvariable==variable)
		self.variable=variable
		self.valorobjetivo=valorobjetivo

	def __call__(self,registro):
		"""Llamando a la funcion establecemos si el registro pasa la condicion de seleccion o no"""
		if registro[self.variable]==self.valorobjetivo:
			return True
		return False
		



class Registro(list):
	"""Almacena un registro, permitiendo el acceso tanto por el indice, como por el nombre de variable. 
	Esta clase garantiza que la lista de valores se ajusta a los tipos de la lista de variables
	Hereda de list"""

	def __init__(self,variables):
		""" 
		Clase constructora
		variables es el array de variables al que esta asociado el registro
		Añade los valores por defecto para todos los atributos
		"""
		#VARIABLES PRIVADAS
		self.__variables=variables

		list.__init__(self)
		for x in self.__variables.arr_defaults():
			self.append(x)	
	
	#FUNCIONES PUBLICAS
	def __getitem__(self,indice):
		""" Redefinicion del [] de la lista. 
		Si nos pasan una ristra, comprobamos si se corresponde a alguna variable
		Si nos pasan un objeto, comprobamos si se corresponde a alguna variable
		Si nos pasan un entero, devolvemos el valor de esa posicion
		"""
		miindice=self.__variables.obtenerindice(indice)
		return list.__getitem__(self,miindice)

	def __setitem__(self,indice,valor):
		""" 
		Redefinicion de []= de la lista.  Almacena en el valor convertido a formato interno por la variable asociada
		"""
		miindice=self.__variables.obtenerindice(indice)
		if not isinstance(valor,str):
			valor=repr(valor)
		list.__setitem__(self,miindice,self.__variables[miindice].nuevoitem(valor))

	def append(self,valor):
		""" 
		Redefinicion del append de lista. Añade al registro el valor pasado en el formato interno de la variable asociada 
		"""
		#TODO: Este append es problemático, puesto que depende de que la variable ya exista y ademas sea la última. Pendiente de rediseñar
		intermedio=self.__variables[self.__len__()].nuevoitem(valor)
		list.append(self,intermedio)
		
	def set(self,lista):
		""" Esta funcion establece los valores del registro, teniendo como argumento una lista o derivado """
		i=0
		for valor in lista:
			self.__setitem__(i,valor)
			i+=1


class ListaVar(list): 
	"""ListaVar hereda de list. Su utilidad es almacenar un conjunto de variables, permitiendo recuperarlas no solo por el indice, sino por el nombre."""

	def __getcopy__(self):
		resultado=ListaVar()
		for i in self:
			resultado.append(i)
		return resultado

	#FUNCIONES PUBLICAS

	def arr_defaults(self):
		"""Devuelve una lista con las ristras de todos los valores por defecto"""
		return [x.valorpordefecto for x in self]

	def obtenerindice(self,parametro):
		"""
 		Permite obtener el indice de una variable a partir de su nombre ,del objeto, o de un entero.
		Devuelve un entero.
		"""
		if isinstance(parametro,QString):
			parametro=parametro.latin1()

		if parametro.__class__==types.StringType:
			for iter in self:  
				if iter.nombre == parametro:
					return self.index(iter)
			return False
		elif parametro.__class__==types.IntType:
			return parametro
		else: #El parametro lo tomamos como una referencia
			if parametro in self:
				return self.index(parametro)
			return False


class ContenedorRegVar:
	"""Esta clase almacena los datos del programa. Tiene dos miembros, una lista de registros y una lista de variables"""
	def __init__(self,registros=None,variables=None,flagoriginal=False):
		"""Inicialización del contenedor principal"""
		self.flagoriginal=flagoriginal
		if not registros: 
			registros=[]
		self.registros=registros #Un array para datos
		if not variables: 
			variables=ListaVar()
		self.variables=variables #Un array de variables 

	def __deepcopy__(self,memo):
		"""Reimplementación de __deepcopy__, permite una copia del objeto"""
		nuevalista=self.variables.__getcopy__()
		nuevalistaregistros=list()
		for registro in self.registros:
			nuevoregistro=Registro(nuevalista)
			for x in range(len(registro)):
				nuevoregistro[x]=nuevalista[x].nuevoitem(repr(registro[x]))
			nuevalistaregistros.append(nuevoregistro)
		resultado= ContenedorRegVar(nuevalistaregistros,nuevalista,False)
		memo[id(self)]=resultado
		return resultado




class ListaUndoRedo(list):
	"""ListaUndoRedo hereda de lista. Es similar a una cola lifo"""
	def __init__(self,tama):
		self.tama=tama

	def encolar(self,objeto,flagoriginal=False,index=0):
		"""Si pasan un indice es que quieren encolar desde alguna posición"""
		if index>=self.tama:
			pass #error
		else:
			if flagoriginal:
				objeto.flagoriginal=True
			if self.__len__()==self.tama:
				self.__delitem__(-1)
			self.__delslice__(0,index)
			self.insert(0,objeto)

class PorteroDatos:
	"""Clase de portera. Gestiona una lista con todas las copias de los datos, determinando cual es la vigente y permitiendo el undo y el redo"""
	def __init__(self):
		self.__index=0 #Variable que lleva el indice
		self.lista=ListaUndoRedo(5)
		self.lista.encolar(ContenedorRegVar(),True)

	def undo(self):
		"""Realiza un undo, desplazando el indice de ListaUndoRedo"""
		if self.__index<self.lista.tama:
			self.__index+=1

	def redo(self):
		"""Realiza un redo, desplazando el indice de ListaUndoRedo"""
		if self.__index>0:
			self.__index-=1

	def puedoundo(self):
		"""Devuelve un valor lógico indicando si es posible realizar un undo"""
		return ((self.__index<self.lista.tama) and (len(self.lista)>1) and (self.__index<(len(self.lista)-1)))

	def puedoredo(self):
		"""Devuelve un valor lógico indicando si es posible realizar un redo"""
		return self.__index>0

	def guardarestado(self,flagoriginal=False):
		"""guarda un estado en la lista de estados, creando una copia a partir del último
		La flagoriginal se activa si se esta abriendo un nuevo fichero
		"""
		tmp=self.lista[self.__index]
		import copy
		objeto= copy.deepcopy(tmp)
		self.lista.encolar(objeto,flagoriginal,self.__index)
	
	def insertarestado(self,estado,flagoriginal=False):
		"""Introduce un estado y lo encola"""
		self.lista.encolar(estado,flagoriginal,self.__index)

	def actual(self):
		"""Devuelve el estado vigente"""
		return self.lista[self.__index]

class InterfazDatos:
	"""Esta clase tiene como objetivo almacenar los datos que seran manipulados en la ejecución del programa.
	Estos datos son un conjunto de variables, y un conjunto de registros que se ajustan a dichas variables.
	La clase ofrece multiples formas de accesso a los datos internos"""

	def __init__(self,portero,gestorfiltro,gestoragrupacion,gestorpaquetes):
		"""Inicialización del contenedor principal"""
		self.__portero=portero
		self.__gestorfiltro=gestorfiltro
		self.__gestoragrupacion=gestoragrupacion
		self.__gestorpaquetes=gestorpaquetes

	#FUNCIONES PUBLICAS

	def __getitem__(self,indice): #Acceso por []
		"""Devuelve el item segun un indice numerico, un nombre, o un objeto variable
			Si el indice es un indice, se entiende que se quiere acceder a un registro
			Si el indice es un nombre o el objeto variable, se entiende que se quiere acceder a la columna
		"""
		if indice.__class__==types.IntType:
			return self.__portero.actual().registros[indice]
		else:
			return self.getCol(indice,filtrado=True)
			
	def __setitem__(self,indice,valor): 
		"""
			Modificacion por []
			Si el indice es un entero, se entiende que se quiere modificar a un registro
			Si el indice es un nombre o el objeto variable, se entiende que se quiere modificar a la columna
		"""
		if indice.__class__==types.IntType:
			self.__portero.actual().registros[indice]=valor
		else: 
			#Añade los registros que no existen
			for registro in range(len(valor)-len(self.__portero.actual().registros)):
				self.appReg()

			for registro in range(len(self.__portero.actual().registros)):
				self.__portero.actual().registros[registro][indice]=valor[registro]

	def __delitem__(self,indice):
		"""define el borrado con la función del, borra un registro"""
		if indice.__class__==types.IntType:
			self.__portero.actual().registros.__delitem__(indice)

	def borrartodo(self):
		"""Borra todos los datos"""
		del self.__portero.actual().registros[0:self.numReg()]
		del self.__portero.actual().variables[0:self.numVar()]
		self.__portero.actual().flagoriginal=True

	def getVar(self,indice=-1):
		"""Devuelve la variable indicada por el indice, por defecto la última"""
		return self.__portero.actual().variables[self.__portero.actual().variables.obtenerindice(indice)] 

	def setVar(self,indice,variable,columna=None):
		"""Sobreescribe la variable asociada al indice.
			Hace una conversión de tipos según la representación interna de estos
			Si se le pasa la columna, entonces sobreescribira todos los registros asociados a la variable
			"""
		#Debe comprobar la correctitud del array y lanzar una excepcion si no vale
		array=[]
		if variable.nombre in self.arrTit(factores=True):
			from  Driza.excepciones import VariableExisteException
			raise VariableExisteException()
		if not columna:
			#Conversion directa (Inestable)
			for elemento in self.getCol(indice):
				array.append(repr(elemento))
			self.__portero.actual().variables[indice]=variable
			for registro in self.__portero.actual().registros:
				registro[indice]=variable.nuevoitem(array[indice])

		else:
			#Nos pasan la columna con los datos
			self.__portero.actual().variables[indice]=variable
			for registro in self.__portero.actual().registros:
				registro[indice]=variable.nuevoitem(columna[indice])
		#TODO: Continuar con el caso de la columna

	def numReg(self):
		"""Devuelve el número de registros almacenados"""
		return len(self.__portero.actual().registros)

	def numVar(self):
		"""Devuelve el número de variables almacenadas"""
		return len(self.__portero.actual().variables)

	def appReg(self,dato=None):
		"""Añade un dato.
		acepta tanto un objeto registro como un objeto lista """
		if isinstance(dato,Registro):
			self.__portero.actual().registros.append(dato)
		else:
			registro=Registro(self.__portero.actual().variables)
			if dato.__class__==types.ListType: #Controlar otros casos
				registro.set(dato)
			self.__portero.actual().registros.append(registro)

	def insReg(self,pos,dato):
		"""inserta un registro en la posicion pos"""
		if isinstance(dato,Registro):
			self.__portero.actual().registros.insert(pos,dato)
		else:
			registro=Registro(self.__portero.actual().variables)
			if dato.__class__==types.ListType: #Controlar otros casos
				registro.set(dato)
			self.__portero.actual().registros.insert(pos,registro)
		

	def appVar(self,nombre=None,tipo="Entero",valores={},valorpordefecto=0,descripcion="",expresion=None):  
		"""Añade una variable de tipo tipo al array de variables. 
		Rellena los registros con el valor por defecto, salvo que se indique una expresion
		"""
		if not nombre:
			nombre="VAR"+str(self.numVar())
		if nombre in self.arrTit(factores=True):
			from  Driza.excepciones import VariableExisteException
			raise VariableExisteException()
		if tipo=="Entero":
			self.__portero.actual().variables.append(variables.TEntero(nombre,valores,valorpordefecto,descripcion)) 
		elif tipo=="Booleano":
			self.__portero.actual().variables.append(variables.TBooleano(nombre,valores,valorpordefecto,descripcion)) 
		elif tipo=="Real":
			self.__portero.actual().variables.append(variables.TReal(nombre,valores,valorpordefecto,descripcion)) 
		elif tipo=="Fecha":
			self.__portero.actual().variables.append(variables.TFecha(nombre,valores,valorpordefecto,descripcion)) 
		elif tipo=="Factor":
			self.__portero.actual().variables.append(variables.TFactor(nombre,descripcion)) 
		else:
			raise TypeError #PENDIENTE: Averiguar el tipo de excepción adecuado
		if expresion:
			import string
			listavars=self.__portero.actual().variables
			#Evaluamos las variables con el nombre más grande para evitar los casos en los que una variable contenga a otra
			listavars.sort(lambda x,y: len(x.nombre)>len(y.nombre))
			listavars.reverse() 
			for var in listavars:
				numero=self.__portero.actual().variables.obtenerindice(var)
				expresion=expresion.replace(var.nombre,"self.__getitem__(i)["+repr(numero)+"]")

			lista=self.__gestorpaquetes.listafunciones()
			cosatmp=self.__gestorpaquetes
			for funcion in lista: #Recorremos la lista de funciones, y hacemos la sustitucion
				print funcion
				texto=string.split(funcion,".")
				ristra="cosatmp[\""+texto[0]+"\"][\""+texto[1]+"\"]"
				expresion=expresion.replace(funcion,ristra)
			for i in range(self.numReg()):
				resultado=eval(expresion)
				self.__portero.actual().registros[i].append(resultado) 
		else: #No nos pasan expresion, rellenamos con los valores por defecto
			for i in range(self.numReg()):
				self.__portero.actual().registros[i].append(valorpordefecto)

	def arrTit(self,tipo=None,factores=True):
		"""
		Devuelve una lista con todos los nombres de las variables
		opcionalmente podemos filtrar segun un tipo
		"""
		if not tipo:
			if factores:
				return [variable.nombre for variable in self.__portero.actual().variables]
			else:
				lista=[]
				for variable in self.__portero.actual().variables:
					if variable.tipo!="Factor":
						lista.append(variable.nombre)
				return lista

		else:
			lista=[]
			for variable in self.__portero.actual().variables:
				if variable.tipo==tipo:
					lista.append(variable.nombre)
			return lista
				
	def arrVar(self,factores=True):
		"""Devuelve la lista con todas las variables"""
		if factores:
			return [var for var in self.__portero.actual().variables]
		else:
			lista=[]
			for variable in self.__portero.actual().variables:
				if variable.tipo!="Factor":
					lista.append(variable)
			return lista

	def getCasos(self,indice=None):
		"""Devuelve todos los casos existentes de una determinada variable"""
		lista=[]
		if not indice:
			indice=self.__gestoragrupacion.variable
		indiceentero=self.__portero.actual().variables.obtenerindice(indice)
		for valor in self.__portero.actual().registros:
			if valor[indiceentero] not in lista:
				lista.append(valor[indiceentero])
		lista.sort()
		return lista

	def getCol(self,indice,filtrado=True,caso=False,condiciones=[]):
		"""Devuelve un array con todos los valores asociados a la variable representada por indice 
		los parametros filtrado y caso son los referidos al filtro general y a la variable de agrupacion general
		"""
		lista=[]
		if not caso:
			#No hay caso
			if not filtrado or not self.__gestorfiltro.variable:
				#No Hay filtrado
				for valor in self.__portero.actual().registros:
					esvalido=True
					if not valor[self.__portero.actual().variables.obtenerindice(indice)]:
						esvalido=False
					for condicion in condiciones:
						if not condicion(valor): #Tiene que verificar todas las condiciones
							esvalido=False 
					if esvalido:
						lista.append(valor[self.__portero.actual().variables.obtenerindice(indice)])
			else:
				#Hay filtrado
				for valor in self.__portero.actual().registros:
					esvalido=True
					if not valor[self.__portero.actual().variables.obtenerindice(indice)]:
						esvalido=False
					for condicion in condiciones:
						if not condicion(valor): #Tiene que verificar todas las condiciones
							esvalido=False 
					if not valor[self.__portero.actual().variables.obtenerindice(self.__gestorfiltro.variable)]:
						esvalido=False #Esta filtrado
					if esvalido:
						lista.append(valor[self.__portero.actual().variables.obtenerindice(indice)])

		else:
			#Hay caso
		#	assert self.__portero.actual().obtenerindice(indice)!=self.__portero.actual().obtenerindice(self.__gestoragrupacion.variable)
			#El caso debe pertenecer a una variable diferente a la que representa el indice
			if not filtrado or not self.__gestorfiltro.variable:
				#No hay filtrado
				for valor in self.__portero.actual().registros:
					esvalido=True
					if not valor[self.__portero.actual().variables.obtenerindice(indice)]:
						esvalido=False
					for condicion in condiciones:
						if not condicion(valor): #Tiene que verificar todas las condiciones
							esvalido=False 
					if caso!=valor[self.__portero.actual().variables.obtenerindice(self.__gestoragrupacion.variable)]:
						esvalido=False
					if esvalido:
						lista.append(valor[self.__portero.actual().variables.obtenerindice(indice)])

			else:
				#Hay filtrado
				for valor in self.__portero.actual().registros:
					esvalido=True
					if not valor[self.__portero.actual().variables.obtenerindice(indice)]:
						esvalido=False
					for condicion in condiciones:
						if not condicion(valor): #Tiene que verificar todas las condiciones
							esvalido=False 
					if not valor[self.__portero.actual().variables.obtenerindice(self.__gestorfiltro.variable)]:
						esvalido=False #Esta filtrado
					if caso!=valor[self.__portero.actual().variables.obtenerindice(self.__gestoragrupacion.variable)]:
						esvalido=False
					if esvalido:
						lista.append(valor[self.__portero.actual().variables.obtenerindice(indice)])

		return lista

	def query(self,variable,caso,*condiciones):
		"""El query es una peticion de datos que sera enviada a R. Hace prácticamente lo mismo que getCol, salvo que en este caso la lista de datos es devuelta directamente en formato float, y acepta condiciones de filtrado"""
		#TODO assert la variable de filtrado es distinta de la de alguna condicion
		#TODO assert la variable de agrupacion es distinta de la de alguna condicion
		#TODO assert la variable es distinta a la de alguna condicion
		mifuncion=lambda f: ParseCondicion(self,f)
		listacondiciones=map(mifuncion,condiciones)
		lista=self.getCol(variable,True,caso,listacondiciones)
		if self.getVar(variable).tipo!="Factor":
			return [float(x) for x in lista]
		else:
			return [ x for x in lista]

	def RData(self):
		"""Devuelve una copia de los datos actuales en el formato data de R"""
		#TODO: Falta el control de casos y filtrado
		import rpy
		diccionario={}
		for variable in self.__portero.actual().variables:
			if variable.tipo!="Factor":
				lista=map(float,self.__getitem__(variable))
			else:
				lista=map(str,self.__getitem__(variable))
			diccionario[variable.nombre]=lista
		df = rpy.r.data_frame(diccionario)
		return df

	def insertarestado(self,estado,flagoriginal=False):
		"""Inserta un estado tal cual"""
		self.__portero.insertarestado(estado,flagoriginal=flagoriginal)

	def agrupacion(self):
		"""Determina si existe una variable de agrupacion o no"""
		if self.__gestoragrupacion.variable:
			return True
		else:
			return False

	def modVar(self,variable,atributo,valor):
		"""Modifica una variable existente. Es necesario para verificar que el nombre no esta ya ocupado
		Variable es el objeto variable, no el indice
		"""
		#FIXME: Estandarizacion de atributos de interfazdato
		if atributo==0:
			if valor in self.arrTit(factores=True):
				from  Driza.excepciones import VariableExisteException
				raise VariableExisteException()
		variable[atributo]=valor

	def original(self):
		"""Devuelve True o False en funcion de si el estado actual es original"""
		return self.__portero.actual().flagoriginal

	def setFlagoriginal(self):
		"""Establece la flag original"""
		self.__portero.actual().flagoriginal=True
