#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#Gestores.py

#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from Driza.clasesbase import OperacionCalculo
from Driza.clasesbase import OperacionDistribucion
from Driza.clasesbase import OperacionGraficos
from Driza.clasesbase import OperacionTabla
from qt import *



class GestorOperaciones(dict):
	"""Gestiona las operaciones que están activas en la ejecución del programa"""
	def __init__(self,dato):
		self.lista=list()
		self.__dato=dato
		self.__initintegrado()

	
	def __initintegrado(self):
		"""Esta funcion carga en el diccionario las operaciones"""
		#Sacado de: http://www.diveintopython.org/functional_programming/all_together.html
		import Driza.operaciones
		ruta=Driza.operaciones.__path__[0]
		import os
		import re
		ficheros=os.listdir(ruta)
		del ficheros[ficheros.index("__init__.py")]
		test = re.compile(".*\.py$", re.IGNORECASE)
		ficheros = filter(test.search, ficheros)   
		filenameToModuleName = lambda f: os.path.splitext(f)[0]
		moduleNames = map(filenameToModuleName, ficheros)
		mifuncion=lambda f: "Driza.operaciones."+f
		moduleNames = map(mifuncion, moduleNames)
		mifuncion2=lambda f: __import__(f,None,None,["Driza.operaciones"])
		modules = map(mifuncion2, moduleNames) 
		for operacion in modules:
			try:
				tipo=operacion.tipo
				if tipo=="Calculo":
					mioperacion=OperacionCalculo(operacion.nombre,self.__dato,operacion.widget,operacion.funcion,operacion.interfazresultado,operacion.initresultado)	
				elif tipo=="Distribucion":
					mioperacion=OperacionDistribucion(operacion.nombre,self.__dato,operacion.widget,operacion.funcion,operacion.interfazresultado,operacion.initresultado)	
				elif tipo=="Graficos":
					mioperacion=OperacionGraficos(operacion.nombre,self.__dato,operacion.widget,operacion.funcion,operacion.interfazresultado,operacion.initresultado)	
				elif tipo=="Tabla":
					mioperacion=OperacionTabla(operacion.nombre,self.__dato,operacion.widget,operacion.funcion,operacion.interfazresultado,operacion.initresultado)	
				self.__setitem__(operacion.nombre,mioperacion)

			except:
				print u"Módulo erróneo"

class Puntero:
	"""Clase que sirve a otras clases para acceder a la clase que actualmente esta desempeñando una funcion"""
	def __init__(self):
		self.variable=None
	def set(self,variable):
		self.variable=variable
	def unset(self):
		self.variable=None


class PunteroFiltro(Puntero):
	"""Almacena la variable que actua como filtro"""
	def set(self,filtro):
		from Driza.variables import TBooleano
		assert isinstance(filtro,TBooleano)
		Puntero.set(self,filtro)
	

class PunteroAgrupacion(Puntero):
	"""Almacena la variable que actua como Agrupacion"""
	def set(self,agrupacion):
		from Driza.variables import TEntero
		assert isinstance(agrupacion,TEntero)
		Puntero.set(self,agrupacion)

class PunteroConfig(Puntero):
	def set(self,config):
		from Driza.clasesbase import Configuracion
		assert isinstance(config,Configuracion)
		Puntero.set(self,config)


class GestorFicheros:
	"""Almacena la variable que actua como fichero.  Dispone además de funciones de carga y almacenamiento"""
	def __init__(self,portero,interfazdato,config):
		from Driza.interfaz import InterfazFichero
		self.fichero=None
		self.__config=config
		self.__portero=portero
		self.__if=InterfazFichero(interfazdato)


	def guardar(self,fichero=None,sobreescribir=False):
		"""Guarda la copia actual de los datos en el fichero indicado"""
		if fichero: self.fichero=fichero
		import os
		if not sobreescribir and os.path.exists(self.fichero): #El fichero existe
			from Driza.excepciones import FicheroExisteException
			raise FicheroExisteException(self.fichero)
		else:
			self.__if.save(self.fichero,self.__portero.actual())
			self.__config.variable["lfichero"].insert(self.fichero)

	def cargar(self,fichero=None):
		"""Carga los datos del fichero indicado"""
		if fichero: self.fichero=fichero
		if not self.fichero:
			pass #Excepcion
		self.__portero.guardarestado()
		self.__if.load(self.fichero)
		self.__config.variable["lfichero"].insert(self.fichero)
		#TODO: Integrar eso

	def importartexto(self,fichero=None,delimitadorreg="\n",delimitadoratrib="\t",cabeceras=False):
		self.__if.importartexto(fichero,delimitadorreg,delimitadoratrib,cabeceras)
		

class GestorTemas:
	"""Gestiona los temas. Su utilidad real queda pendiente de futuros diseños"""
	def __init__(self):
		import os
		self.temaactual=None
		self.ruta=None
		posiblesrutas=["Driza/images/","/usr/lib/python2.4/site-packages/Driza/images/","~/.driza/images/"]
		for ruta in posiblesrutas:
			if os.path.exists(ruta):
				self.ruta=ruta
				break

		if not self.ruta:
			print "ERROR" #TODO: Raise excepción

	def portada(self):
		return QPixmap(self.ruta+"driza.png")

	def iconoabrir(self):
		"""Devuelve un QImage con el icono de abrir"""
		return QPixmap(self.ruta+"botonabrir.png")

	def iconoguardar(self):
		return QPixmap(self.ruta+"botonguardar.png")

	def iconoexportar(self):
		return QPixmap(self.ruta+"botonexportar.png")


class GestorPaquetes(dict):
	"""Gestiona los paquetes que contendran las funciones. Estos paquetes son equivalentes al modulo de python que los contiene"""
	def __init__(self):
		basicas=Paquete("basicas")
		self.__setitem__("basicas",basicas)
		#Intenta cargar el modulo
		#Cargaria todas las funciones de este
		#Leeria una descripción del modulo (Atributo "descripcion" dentro del ficher)
		#Cada funcion tendria un parametro nombre, y otro descripcion

		#La estructura final seria:
		#{PAritmeticas:[FSuma,FResta,etc...],...} #PAritmeticas es un paquete
		# El dialogo o lo que sea consultaria a cada funcion el nombre y la descripcion

	def listafunciones(self):
		"""Devuelve una lista con todos los paquetes y todas las funciones"""
		lista=[]
		for paquete in self.items():
			for funcion in paquete[1].keys():
				lista.append(paquete[1].nombrereal+"."+funcion)
		return lista


class Paquete(dict):
	def __init__(self,nombre):
		"""Carga el paquete de  una ruta"""
		ruta="Driza.funciones."+nombre
		self.nombrereal=nombre # El nombre real del paquete
		mifuncion2=lambda f: __import__(f,None,None,["Driza.funciones"])
		modulo=mifuncion2(ruta)
		listamiembros=dir(modulo)
		import re
		p = re.compile('^[A-Z]+')
		nuevalista=filter(p.match,listamiembros)
		del nuevalista[nuevalista.index("Funcion")]
		nuevalista2=[]
		for x in nuevalista:
			nombre="modulo"+"."+x+"()"
			nuevalista2.append(eval(nombre))
		for x in range(len(nuevalista)):
			self.__setitem__(nuevalista[x],nuevalista2[x]) #Ponemos valor al diccionario

