#!/usr/bin/python
# -*- coding: utf-8 -*-
#Clases de elementos correspondientes a la ventana de salida

#salida.py: Objetos relacionados con la ventana de salida

#Copyright (C) 2006  Inmaculada Luengo Merino, Nstor Arocha Rodrguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


class Resultado(dict):
	"""Esta clase representa un resultado, que es un conjunto de Elementosresultado
	Recibe una lista de diccionarios con los resultados de variables-casos, y los interpreta generando unos elementos
	"""
	def __init__(self):
		self.titulo=""
		dict.__init__(self)

	def addTablaSimple(self,nombre):
		"""El metodo pasado es el que se usara para rellenar el campo a partir del diccionario de entrada"""
		elemento=TablaResultadoSimple()
		self.__setitem__(nombre,elemento)
		
	def addTablaDoble(self,nombre):
		"""El metodo pasado es el que se usara para rellenar el campo a partir del diccionario de entrada"""
		elemento=TablaResultadoDoble()
		self.__setitem__(nombre,elemento)

	def addTexto(self,nombre):
		self.__setitem__(nombre,TextoResultado())

	def addImagen(self,nombre):
		self.__setitem__(nombre,ImagenResultado())

	def html(self):
		"""Genera el html a partir de todos sus componentes + el titulo"""
		ristra=u""
		for elemento in self.values():
			ristra+=elemento.html()
		return ristra
		
	def exportarhtml(self,ruta):
		"""Genera el html a partir de todos sus componentes + el titulo"""
		ristra=u""
		for elemento in self.values():
			ristra+=elemento.exportarhtml(ruta)
		return ristra

class ElementoResultado:
	"""Cualquier elemento que figure en un resultado ser hijo de esta clase"""
	def __init__(self):
		pass

	def set(self,dict):
		"""Abstracta: define como ha de recibir cada elemento la informacin del diccionario dict"""
		pass

	def html(self):
		"""Abstracta: muestra el contenido del widget volcado a html"""
		return ""

	def exportarhtml(self,fichero):
		"""Esta funcion es llamada cuando se desea exportar html a un fichero. Normalmente hace lo mismo que html()"""
		return self.html()


class Tabla:
	"""Almacena una tabla. Esta clase asegura que se cumple la restriccion del tamao para todos los registros"""
	def __init__(self):
		self.__contenido=list()
		self.__ncols=None
		self.__listatitulos=None

	def settitulo(self,args):
		self.__listatitulos=args

	def set(self,args):
		"""Introduce los argumentos en la tabla. Ha de coincidir en tamao con ncols"""
		if not self.__ncols:
			self.__ncols=len(args)
		if len(args) != self.__ncols:
			print "error"
		else:
			self.__contenido.append(args)


	def html(self):
		"""Devuelve el contenido de la tabla en html"""
		cadena=""
		if self.__listatitulos:
			cadena+="<tr>\n"
			for titulo in self.__listatitulos:
				cadena+="<td>"+titulo+"</td>"
			cadena+="</tr>\n"

		for x in self.__contenido:
			cadena+="<tr>\n"
			for y in x:
				cadena+="<td>"+repr(y)+"</td>\n"
			cadena+="</tr>\n"
		return cadena

class TablaResultado(ElementoResultado):
	"""Clase antesesora de todas las tablasresultados"""
	def __init__(self):
		ElementoResultado.__init__(self)
		self.contenido=Tabla()
		self.titulo=""

	def html(self):
		ristra=""
		ristra+="<h2>"+self.titulo+"</h2>"
		ristra+="<table border=\"1\">"
		ristra+=self.contenido.html()
		ristra+="</table>"
		return ristra



class TablaResultadoSimple(TablaResultado):
	"""Una tabla. Convierte un array de arrays en una tabla html"""
	def set(self,variable,listaargs):
		"""Carga los argumentos en la tabla, pero solo los que no sean nulos"""
		variable=variable.latin1()
		self.contenido.set([variable]+listaargs)

	def settitulo(self,titulo):
		"""Estable el titulo de la tabla"""
		self.contenido.settitulo(["Variable"]+titulo)


class TablaResultadoDoble(TablaResultado):
	"""Una tabla. Convierte un array de arrays en una tabla html"""
	def set(self,variable,listaargs):
		"""Carga los argumentos en la tabla, pero solo los que no sean nulos"""
		self.contenido.set([variable[0].latin1(),variable[1].latin1()]+listaargs)

	def settitulo(self,titulo):
		"""Estable el titulo de la tabla"""
		self.contenido.settitulo(["Variable1","Variable2"]+titulo)

class TextoResultado(ElementoResultado):
	"""Es un texto cualquiera"""
	def __init__(self):
		ElementoResultado.__init__(self)
		self.contenido=""
		self.titulo=""

	def set(self,texto):
		self.contenido=str(texto)

	def settitulo(self,titulo):
		self.titulo=titulo

	def html(self):
		if self.contenido:
			return "<h3>"+self.titulo+"</h3>"+self.contenido
		else: 
			return ""

class ImagenResultado(ElementoResultado):
	"""Representa una imagen"""
	def __init__(self):
		ElementoResultado.__init__(self)
		self.contenido=None #TODO
		self.ruta=None

	def set(self,ruta):
		self.ruta=ruta

	def settitulo(self,titulo):
		self.titulo=titulo

	def exportarhtml(self,carpeta):
		"""Copia el archivo a la carpeta donde estaran las imagenes, y devuelve el html con la ruta de la imagen actualizada
		La carpeta ya debe estar creada
		"""
		import os
		import shutil
		ristra=""
		if self.ruta:
			(dirName, fileName) = os.path.split(self.ruta)
			shutil.copyfile(self.ruta,carpeta+"/"+fileName)
			ristra+="<h2>"+self.titulo+"</h2>"
			ristra+="<img src=\""+os.path.split(carpeta)[1]+"/"+fileName+"\">"
		return ristra

	def html(self):
		ristra=""
		if self.ruta:
			ristra+="<h2>"+self.titulo+"</h2>"
			ristra+="<img src=\""+self.ruta+"\">"
		return ristra


