#!/usr/bin/python
# -*- coding: utf-8 -*-
#####
# Fichero de TVariables
# Guarda todos los TVariables relacionados con los TVariables que maneja el programa
####

#Copyright (C) 2006  Inmaculada Luengo Merino, Nstor Arocha Rodrguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

class Variable:
	"""La clase padre de todas las variables"""
	def __init__(self):
		pass
	def __eq__(self,other):
		return self.valor==other.valor
	def __nonzero__(self):
		if self.valor!=None: 
			return True
		else: 
			return False

class VariableNumerica(Variable):
	"""Todas las variables que tienen representacin numrica"""
	def __init__(self,tipovariable,valor):
		Variable.__init__(self)
		self.tvariable=tipovariable
		self.valor=self.interno(valor)
	def __repr__(self):
		pass
	def __float__(self):
		return 0

class Factor(Variable):
	"""Las variables que solo son cadenas y que nicamente sirven para agrupacin"""
	def __init__(self,tfactor,valor):
		Variable.__init__(self)
		self.tvariable=tfactor
		self.valor=self.__interno(valor)

	def __interno(self,cadena):
		try:
			eval(cadena)
		except:
			return cadena
		else:
			return eval(cadena)

	def __str__(self):
		return self.valor
	def __repr__(self):
		try:
			eval(self.valor)
		except:
			return repr(self.valor)
		else:
			return self.valor


class Booleano(VariableNumerica):
	"""Variable booleana. Almacena verdadero o falso"""

	def interno(self,valor):
		"""Transforma una cadena al formato interno, devolviendolo como resultado"""
		if valor==False or valor=="0" or valor=="Falso" or valor=="False":
			return False
		elif valor==True or valor=="1" or valor=="Verdadero" or valor=="True":
			return True
		else:
			raise ValueError

	def __float__(self):
		"""Devuelve la representacion en float del valor interno"""
		if self.valor:
			return 1.0
		else:
			return 0.0

	def __repr__(self):
		"""Devuelve la representacion en ristra del valor interno"""
		if self.valor:
			return "Verdadero"
		else:
			return "Falso"

	def __nonzero__(self):
		"""Devuelve el estado booleano de la variable. Es necesario para las comparaciones de python (if variable: )"""
		if self.valor:
			return True
		else:
			return False

class Real(VariableNumerica):
	def interno(self,valor):
		if valor=="None" or valor=="NA" or valor=="nan" or valor==float("nan"):
			return None
		else:
			return float(valor)
	def __float__(self):
		return self.valor
	def __repr__(self):
		return repr(self.valor)
	def __str__(self):
		return str(self.valor)


class Entero(VariableNumerica):
	def interno(self,valor):
		return int(valor)
	def __float__(self):
		return float(self.valor)
	def __repr__(self):
		return repr(self.valor)

class Fecha(VariableNumerica):
	def interno(self,valor):
		import datetime
		array=valor.split("/")
		fecha=datetime.date(int(array[2]),int(array[1]),int(array[0]))
		import time
		return time.mktime(fecha.timetuple())
	def __float__(self):
		return float(self.valor)
	def __repr__(self):
		import time
		import datetime
		fecha=datetime.datetime.fromtimestamp(self.valor)
		return repr(fecha.day)+"/"+repr(fecha.month)+"/"+repr(fecha.year)


class TVariable:
	"""Clase de tipo que es padre de todos los tipos"""
	def update(self,objeto):
		"""Funcin de copia de los atributos para cualquier TVariable.
		Sirve para hacer copias entre TVariables
		"""
		self.nombre=objeto.nombre
		self.descripcion=objeto.descripcion

class TFactor(TVariable):
	"""Clase de tipo que representa a los factores"""
	def __init__(self,nombre=None,descripcion=""):
		self.valorpordefecto=""
		self.descripcion=descripcion
		self.nombre=nombre
		self.diccionarioconversion={"TVariable":["CDefault"]}
		self.tipo="Factor"

	def __setitem__(self,indice,valor):
		"""Definicion del operador []=. Permite modificar los atributos de la TVariable a travs de un indice numrico"""
		if indice==0:
			self.nombre=valor
		elif indice==4:
			self.descripcion=valor

	def __getitem__(self,indice):
		"""Definicion del operador[]. Permite acceder a los atributos de la TVariable por indices numricos"""
		if indice==0:
			return self.nombre
		elif indice==4:
			return self.descripcion

	def nuevoitem(self,ristra):
		return Factor(self,ristra)

class TVariableNumerica(TVariable):
	"""
	TVariable generica. De esta clase heredan las TVariablesNumericos usados en el programa. Define los elementos comnes a todas las TVariablesNumericas y posee las funciones de acceso genricas
	"""


	def __init__(self,nombre,valores,valorpordefecto,descripcion):
		#TVariableS PUBLICAS
		self.nombre=nombre
		self.valorpordefecto=valorpordefecto
		self.descripcion=descripcion
		if valores == None: self.valores=dict()
		else: self.valores=valores
		self.diccionarioconversion={} #Cada subclase debe incluirlo

	#FUNCIONES PUBLICAS
		
	def update(self,objeto):
		"""Funcin de copia de los atributos para cualquier TVariable.
		Sirve para hacer copias entre TVariables
		"""
		TVariable.update(self,objeto)
		try:
			if objeto.valores:
				self.valores=objeto.valores.copy()
		except:
			print "Error FIXME" #Copiando de un factor

	def __setitem__(self,indice,valor):
		"""Definicion del operador []=. Permite modificar los atributos de la TVariable a travs de un indice numrico"""
		if indice==0:
			self.nombre=valor
		elif indice==3:
			self.valorpordefecto=valor
		elif indice==4:
			self.descripcion=valor

	def __getitem__(self,indice):
		"""Definicion del operador[]. Permite acceder a los atributos de la TVariable por indices numricos"""
		if indice==0:
			return self.nombre
		elif indice==3:
			return self.valorpordefecto
		elif indice==4:
			return self.descripcion
	

class TNumerico(TVariableNumerica):
	pass


class TEntero(TNumerico):
	"""
	La clase entero. Almacena cualquier nmero entero, garantizando las operaciones de transformacin a formato interno y a ristra. Hereda de TVariable
	"""
	def __init__(self,nombre=None,valores=None,valorpordefecto=0,descripcion=None):
		"""
		Inicializacion de TVariable entera
		"""
		TVariableNumerica.__init__(self,nombre,valores,valorpordefecto,descripcion)
		self.tipo="Entero"
		self.diccionarioconversion={"TVariable":["CDefault"]}
	
	def nuevoitem(self,valor):
		return Entero(self,valor)

class TReal(TNumerico):
	"""
	La clase Real. Almacena cualquier nmero real, garantizando las operaciones de transformacin a formato interno y a ristra. Hereda de TVariable
	"""
	def __init__(self,nombre=None,valores=None,valorpordefecto=0,descripcion=None):
		"""Inicializacion del objeto Real"""
		TVariableNumerica.__init__(self,nombre,valores,valorpordefecto,descripcion)
		self.tipo="Real"
		self.diccionarioconversion={"TVariable":["CDefault"]}

	def nuevoitem(self,valor):
		return Real(self,valor)

class TBooleano(TVariableNumerica):
	"""La clase Booleano,representa a un dato que solo puede tener valor verdadero o falso. El almacenamiento en python corresponde a los valores True o False """
	def __init__(self,nombre=None,valores=None,valorpordefecto="0",descripcion=None):
		"""Inicializacin del objeto booleano"""
		TVariableNumerica.__init__(self,nombre,valores,valorpordefecto,descripcion)
		self.tipo="Booleano"
		self.diccionarioconversion={"TVariable":["CDefault"]}

	def nuevoitem(self,valor):
		return Booleano(self,valor)


class TFecha(TVariableNumerica):
	"""
	La clase Fecha. Almacena una fecha en formato DD/MM/YYYY, garantizando las operaciones de transformacin a formato interno y a ristra. Hereda de TVariable
	"""
	def __init__(self,nombre=None,valores=None,valorpordefecto=None,descripcion=None):
		"""Inicializacion de fecha"""
		if valorpordefecto==None:
			valorpordefecto="1/1/1970"
		self.tipo="Fecha"
		TVariableNumerica.__init__(self,nombre,valores,valorpordefecto,descripcion)
		self.diccionarioconversion={"TVariable":["CDefault"]}

	def nuevoitem(self,valor):
		return Fecha(self,valor)

