#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from qt import *
from Driza.ui.ventanas.uivprincipal import vprincipal
from Driza.dialogs.dconfig import DConfig
from Driza.dialogs.dseleccionvar import DFiltro
from Driza.dialogs.dseleccionvar import DAgrupacion
from Driza.dialogs.dcrevar import DCrevar
from Driza.dialogs.dbuscar import DBuscar
from Driza.dialogs.doperaciones import DOperaciones
from Driza.dialogs.dimportartexto import DImportarTexto
from Driza.widgets.grid import Grid

from Driza import listas
import string
from Driza.widgets.widgetsqt import GrisTableItem

class VPrincipal(vprincipal):
	"""Ventana principal, hereda de vprincipal, que esta generado por el designer. Esta clase es padre de la mayoria de los dialogos, y es la que permite introducir los datos a el usuario"""

	def __init__(self,parent,datos,gestoroperaciones,gestorfiltro,config,portero,gestorficheros,gestortemas,gestoragrupacion,gestorpaquetes): 
		"""Inicializador de la ventana principal"""
		vprincipal.__init__(self,None)
		#VARIABLES PUBLICAS
		self.__datos=datos
		self.parent=parent
		self.__config=config
		self.__gestortemas=gestortemas
		#Bloque de inicialización de dialogos
		self.__doperaciones = DOperaciones(self,gestoroperaciones,self.parent.vsalida)
		self.__dcrevar = DCrevar(self,datos,config,gestorpaquetes)
		self.__dfiltro = DFiltro(gestorfiltro,datos,self.__dcrevar,self) #Candidata a no ser miembro
		self.__dagrupacion = DAgrupacion(gestoragrupacion,datos,self.__dcrevar,self)
		self.__dbuscar = DBuscar(self,"busqueda")
		self.__dconfig = DConfig(config,self)
		self.__dimportartexto=DImportarTexto(self,gestorficheros)
		self.__portero=portero
		self.__gestorfichero=gestorficheros
		self.grid= Grid(self,self.__datos,self.__portero,gestorfiltro)
		self.widgetStack1.addWidget(self.grid)
		self.widgetStack1.raiseWidget(self.grid)

		# Funciones de inicializacion de componentes
		self.__inittoolbar()
		self.__conexiones()
		self.showundoredo()

	#FUNCIONES PUBLICAS
	
	def __importar(self):
		"""Importa los datos de un fichero"""
		self.__dimportartexto.show()

	def abrirfichero(self):
		""" Lanza el diálogo de apertura de fichero """
		if not self.__preguntarficheromodificado(): return
		filter=""
		for fmt in listas.extensiones_fichero():
			filter=filter+"%s files (*.%s);;" % (fmt,string.lower(fmt))
		fn = QFileDialog.getOpenFileName(QString.null,filter,self,None,"Dialogo abrir fichero","")
		filename = str(fn)
		if filename:
			self.__gestorfichero.cargar(filename)
		self.show()
		self.grid.show()

	def show(self):
		"""Esta funcion actualiza el contenido de las rejillas segun los datos
		Reimplementa el show de la clase base
		"""
		#Tabla de datos
		if self.__gestorfichero.fichero:
			nfichero= self.__gestorfichero.fichero
		else:
			nfichero= "Nuevo fichero"
		self.setCaption("Driza "+self.parent.version()+" - "+nfichero)
		#Titulo de datos
		vprincipal.show(self)

	def closeEvent( self, ev ):
		"""Acciones que se realizaran cuando el usuario cierre la ventana. Guarda el estado de la configuración, incluyendo los ficheros recientemente abiertos"""
		from Driza.interfaz import InterfazConfig
		if not self.__preguntarficheromodificado(): return
		ic=InterfazConfig()
		ic.save(self.__config.variable)
		QWidget.closeEvent( self, ev )

	#FUNCIONES PRIVADAS
	
	def showundoredo(self):
		"""Determina que botones estan activados y cuales no del submenu de edición"""
		if self.__portero.puedoundo():
			self.editUndoAction.setEnabled(True)
		else:
			self.editUndoAction.setEnabled(False)

		if self.__portero.puedoredo():
			self.editRedoAction.setEnabled(True)
		else:
			self.editRedoAction.setEnabled(False)

	def __preguntarficheromodificado(self):
		"""Pregunta en caso de que haya sido modificado el fichero si desea ser guardado"""
		if not self.__datos.original():
			rc = QMessageBox.information(self,'Atencion:', 'El fichero actual ha sido modificado, desea guardarlo?', 'Guardarlo','No guardarlo','Volver',0,1)
			if rc==0:
				self.__gestorfichero.guardar()
			elif rc==2:
				return False
		return True


	def __dacerca_de(self):
		"""Mensaje acerca de"""
		QMessageBox.about(self,"Acerca de Driza", u"Driza %s es una interfaz estadística\n (C) Néstor Arocha Rodríguez - Tutorizado por Inmaculada luengo Merino\n Distribuido bajo licencia GPL\n----------------------------------\n Betatesters:\n Carlos Mestre Gonzalez\n Luis de Bethencourt Guimerá" % (self.parent.version()))
	
	def __undo(self):
		"""Deshace el último cambio"""
		self.__portero.undo()
		self.grid.show()
		self.showundoredo()
	
	def __redo(self):
		"""Rehace el último cambio"""
		self.__portero.redo()
		self.grid.show()
		self.showundoredo()

	def __inittoolbar(self):
		"""Inicializa la toolbar"""
		self.boton1=QToolButton(self.Toolbar)
		self.boton1.setTextLabel("ABRIR")
	 	iconos=QIconSet()
		iconos.setIconSize(QIconSet.Small,QSize(100,100))
		iconos.setPixmap(self.__gestortemas.iconoabrir(),QIconSet.Small)
#		iconos.setPixmap("images/flechaabajo.png",QIconSet.Automatic,QIconSet.Normal,QIconSet.Off)
		self.boton1.setOn(True)
		self.boton1.setIconSet(iconos)
		self.boton2=QToolButton(self.Toolbar)
		self.boton2.setTextLabel("GUARDAR")
	 	iconos=QIconSet()
		iconos.setIconSize(QIconSet.Small,QSize(100,100))
		iconos.setPixmap(self.__gestortemas.iconoguardar(),QIconSet.Small)
		self.boton2.setOn(True)
		self.boton2.setIconSet(iconos)

	def __conexiones(self):
		"""Funcion llamada en el constructor que almacena todas las conexiones accion-funcion"""
		self.connect(self.fileExitAction,SIGNAL("activated()"),self.__salirprograma)
		self.connect(self.archivoSalirAction,SIGNAL("activated()"),self.__salirprograma)
		self.connect(self.fileNewAction,SIGNAL("activated()"),self.__nuevo)
		self.connect(self.fileOpenAction,SIGNAL("activated()"),self.abrirfichero)
		self.connect(self.fileSaveAsAction,SIGNAL("activated()"),self.__guardarcomo)
		self.connect(self.fileSaveAction,SIGNAL("activated()"),self.__guardar)
		self.connect(self.archivoImportarAction,SIGNAL("activated()"),self.__importar)
		self.connect(self.programaConfigurarAction,SIGNAL("activated()"),self.__dconfig.show)
		self.connect(self.datosFiltrarAction,SIGNAL("activated()"),self.__dfiltro.show)
		self.connect(self.datosAgruparAction,SIGNAL("activated()"),self.__dagrupacion.show)
		self.connect(self.modificarCrear_nuevas_variablesAction,SIGNAL("activated()"),self.__dcrevar.show)
		self.connect(self.distribucionesDistribucion_NormalAction,SIGNAL("activated()"),self.__ddistribucionnormal)
		self.connect(self.editFindAction,SIGNAL("activated()"),self.__dbuscar.show)
		self.connect(self.editCopyAction,SIGNAL("activated()"),self.__copy)
		self.connect(self.editCutAction,SIGNAL("activated()"),self.__cut)
		self.connect(self.editPasteAction,SIGNAL("activated()"),self.__paste)
		self.connect(self.editUndoAction,SIGNAL("activated()"),self.__undo)
		self.connect(self.editRedoAction,SIGNAL("activated()"),self.__redo)
		self.connect(self.acerca_deAcerca_deAction,SIGNAL("activated()"),self.__dacerca_de)
		self.connect(self.boton1,SIGNAL("clicked()"),self.abrirfichero)
		self.connect(self.boton2,SIGNAL("clicked()"),self.__guardar)
		self.__conexionesanalizar()


	def __conexionesanalizar(self):
		"""Las conexiones del submenu "analizar" """
		self.connect(self.analizarMediaGrficosHistogramaAction,SIGNAL("activated()"),self.__dgraficoshistograma)
		self.connect(self.analizarMediaGrficosDiagrama_de_dispersinAction,SIGNAL("activated()"),self.__dgraficosdispersion)
		self.connect(self.analizarMediaDescriptivosAction,SIGNAL("activated()"),self.__dcalculodescriptivo)
		self.connect(self.analizarBondad_de_ajusteJi_cuadradoAction,SIGNAL("activated()"),self.__dcalculojicuadrado)
		self.connect(self.AnalizarKs,SIGNAL("activated()"),self.__dcalculoks)
		self.connect(self.analizarRegresin_linealRegresin_lineal_simpleAction,SIGNAL("activated()"),self.__dcalculoregresion)
		self.connect(self._2Comparacin_con_una_cteAction,SIGNAL("activated()"),self.__dcalculocompmedsim)
		self.connect(self._2Comparacin_entre_dos_variablesAction,SIGNAL("activated()"),self.__dcalculocompmeddob)
		self.connect(self.analizarPruebas_de_hiptesisProporcionesComparacin_con_una_cteAction,SIGNAL("activated()"),self.__dcalculocomppropsim)
		self.connect(self.analizarPruebas_de_hiptesisVarianzaComparacin_entre_dos_variablesAction,SIGNAL("activated()"),self.__dcalculocompvardob)
		self.connect(self.analizarMediaTabla_de_frecuenciasAction,SIGNAL("activated()"),self.__dtablafrecuencia)
		self.connect(self.analizarMediaGrficosDiagrama_de_barrasAction,SIGNAL("activated()"),self.__ddiagramabarras)
		self.connect(self._2ANOVAAction,SIGNAL("activated()"),self.__danova)

	def __copy(self):
		"""Funcion que copia y borra la seleccion"""
		try:
			self.__copiar()
		except:
			QMessageBox.warning(self,u'Atención', 'Las operaciones de copiado, cortado y pegado no han sido implementadas')
		else:
			self.grid.removeselection() 

	
	def __cut(self):
		"""Copia, borra la seleccion y su contenido"""
		self.__portero.guardarestado()
		try:
			self.__copiar()
		except:
			QMessageBox.warning(self,u'Atención', 'Las operaciones de copiado, cortado y pegado no han sido implementadas')
		else:
			self.grid.removeselection(borrardatos=True)
			self.grid.show()


	def __paste(self):
		"""Pega lo que haya sido copiado"""
		self.__portero.guardarestado()
		cb = QApplication.clipboard()
		registro=cb.text().latin1()
		lista=eval(registro)
		pos=self.grid.table1.currentRow()
		lista.reverse() #Escribimos los registros al reves para que queden ordenados
		if pos > self.__datos.numReg():
			for x in range(self.__datos.numReg(),pos):
				self.__datos.appReg()
		#Hacemos el pegado efectivo
		for registro in lista:
			self.__datos.insReg(pos,registro)
		self.grid.show()


	def __salirprograma(self):
		"""Funcion de salida del programa. Guarda la configuracion"""
		from Driza.interfaz import InterfazConfig
		if not self.__preguntarficheromodificado(): return
		ic=InterfazConfig()
		ic.save(self.__config.variable)
		qApp.exit(0)


	def __nuevo(self):
		"""Accion realizada cuando el usuario clickea en Nuevo"""
		if not self.__preguntarficheromodificado(): return
		if self.__gestorfichero.fichero:
			#preguntar si quieres guardar
			#TODO: Verificar si ha sido modificado. 
			self.__datos.borrartodo()
			self.__gestorfichero.fichero=None
			self.show()
			self.grid.show()
		else:
			self.__datos.borrartodo()
			self.__gestorfichero.fichero=None
			self.grid.show()
			self.show()


	
	def __guardar(self):
		"""Funcion de guardado del fichero. Si el fichero no es conocido se llama a guardarcomo"""
		if self.__gestorfichero.fichero:
			self.__gestorfichero.guardar(sobreescribir=True)
			self.__datos.setFlagoriginal()
		else:
			self.__guardarcomo()

	def __guardarcomo(self):
		"""Abre un diálogo pidiendo el nombre de archivo y guarda en dicho archivo"""
		filter=""
		for fmt in listas.extensiones_fichero():
			filter=filter+"%s files (*.%s);;" % (fmt,string.lower(fmt))
		fn = QFileDialog.getSaveFileName(QString.null,filter,self)
		filename = str(fn)
		if filename:
			from Driza.excepciones import FicheroExisteException
			try:
				self.__gestorfichero.guardar(filename)
				self.__datos.setFlagoriginal()
				self.show()
			except FicheroExisteException,fichero:
				rc = QMessageBox.information(self,'Atencion:', 'El fichero'+fichero.fichero+' ya existe' , 'Sobreescribir','Otro nombre','Cancelar',0,1)
				if rc==0:
					self.__gestorfichero.guardar(filename,True)
					self.__datos.setFlagoriginal()
					self.show()
				elif rc==1:
					self.__guardarcomo()
					
	def __copiar(self):
		"""Esta funcion pega el contenido del clipboard en la tabla actual. Es llamada por copy"""
		tablaactual=self.grid.currentPageIndex()
		assert(tablaactual == 0) #Cambiar por una excepcion más concreta
		lista=self.grid.listaseleccion()
		cadena=repr(lista)
		cb = QApplication.clipboard()
		cb.setData(QTextDrag(cadena))
			

	def __dcalculodescriptivo(self):
		"""Muestra el dialogo de calculo descriptivo"""
		self.__doperaciones.show("Descriptivo")

	def __dtablafrecuencia(self):
		"""Muestra"""
		self.__doperaciones.show(u"Tabla de frecuencias")

	def __dcalculojicuadrado(self):
		"""Muestra el dialogo ji cuadrado"""
		self.__doperaciones.show("Ji Cuadrado")

	def __ddiagramabarras(self):
		"""Muestra"""
		self.__doperaciones.show(u"Diagrama de barras")

	def __dcalculoks(self):
		"""Muestra el dialogo ks"""
		self.__doperaciones.show("KS")

	def __dcalculoregresion(self):
		"""Muestra el dialogo de regresion"""
		self.__doperaciones.show(u"Regresión")

	def __dgraficoshistograma(self):
		"""Muestra el la seccion de creación de histogramas"""
		self.__doperaciones.show("Histograma")

	def __dgraficosdispersion(self):
		"""Muestra el la seccion de creación de histogramas"""
		self.__doperaciones.show(u"Diagrama de dispersión")

	def __ddistribucionnormal(self):
		"""Muestra la distribucion normal"""
		self.__doperaciones.show(u"Distribución Normal")

	def __dcalculocompmedsim(self):
		self.__doperaciones.show(u"Comparación de medias simple")
		
	def __dcalculocompmeddob(self):
		self.__doperaciones.show(u"Comparación de medias entre dos variables")

	def __dcalculocomppropsim(self):
		self.__doperaciones.show(u"Comparación de proporciones simple")
		
	def __dcalculocomppropdob(self):
		self.__doperaciones.show(u"Comparación de proporciones entre dos variables")

	def __dcalculocompvardob(self):
		self.__doperaciones.show(u"Comparación de Varianzas entre dos variables")

	def __danova(self):
		self.__doperaciones.show(u"Anova Simple")
