#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from qt import *
from qttable import *
from Driza.variables import * #Para poder preguntar por todos los tipos
from Driza.dialogs.dcasos import DCasos


class AgenteConversion:
	"""Esta clase realiza una conversion de un tipo a otro, devolviendo la nueva variable y la lista de campos de registros ya transformados"""
	def __init__(self,interfazdatos):
		self.__interfazdatos=interfazdatos

	def __call__(self,variable,tipovariabledestino,metododeseado):
		from Driza import variables
		from Driza.conversion import CDefault
		if tipovariabledestino == "Entero":
			nuevavariable=variables.TEntero()
		elif tipovariabledestino == "Real":
			nuevavariable=variables.TReal()
		elif tipovariabledestino == "Booleano":
			nuevavariable=variables.TBooleano()
		elif tipovariabledestino == "Fecha":
			nuevavariable=variables.TFecha()
		elif tipovariabledestino == "Factor":
			nuevavariable=variables.TFactor()
		nuevavariable.update(variable) #Copia de atributos de la variable
		lista=self.__interfazdatos.getCol(variable,filtrado=False)
		tipofuncion=eval(metododeseado)
		lafuncion=tipofuncion(variable,nuevavariable)
		nuevalista=map(lafuncion,lista)
		return [nuevavariable,nuevalista]


class  Grid(QTabWidget):
	"""La tabla que manejara los registros y las variables"""


	#Funciones de inicializacion

	def __init__(self,padre,interfazdatos,porterodatos,gestorfiltro):
		"""Inicializacion"""
		self.__portero=porterodatos
		self.__gestorfiltro=gestorfiltro
		QTabWidget.__init__(self,padre,"Grid")
		self.setTabPosition(QTabWidget.Bottom)
		self.tab = QWidget(self,"tab")
		tabLayout = QVBoxLayout(self.tab,11,6,"tabLayout")
		self.table1 = QTable(self.tab,"table1")
		tabLayout.addWidget(self.table1)
		self.insertTab(self.tab,QString.fromLatin1("Registros"))
		self.tab_2 = QWidget(self,"tab_2")
		tabLayout_2 = QVBoxLayout(self.tab_2,11,6,"tabLayout_2")
		self.table2 = QTable(self.tab_2,"table2")
		tabLayout_2.addWidget(self.table2)
		self.insertTab(self.tab_2,QString.fromLatin1("Variables"))

		self.__nreg=10 
		self.__nvar=10
		self.__interfazdatos=interfazdatos
		self.__inittabladat()
		self.__inittablavar()
		self.__conexiones()
		self.__dcasos=DCasos(self,interfazdatos)#UElementos de la tabla de variables querran acceder
		self.__agenteconversion=AgenteConversion(self.__interfazdatos)
		

	def __inittabladat(self):
		"""Inicializa la tabla de datos"""
		self.table1.setSelectionMode(QTable.MultiRow)
		self.table1.setNumCols(16) 
		self.table1.setNumRows(64)
		i=0
		for x in range(self.table1.horizontalHeader().count()):
			self.table1.horizontalHeader().setLabel(i,'')
			i+=1


	def __inittablavar(self):
		"""Inicializa la tabla de variables"""
		self.table2.setSelectionMode(QTable.MultiRow)
		self.table2.setNumRows(self.__nvar)
		self.table2.setNumCols(5)
		titulos=self.table2.horizontalHeader()
		titulos.setLabel (0,"Nombre")
		titulos.setLabel (1,"Tipo")
		titulos.setLabel (2,"Valores")
		titulos.setLabel (3,"Valor por defecto")
		titulos.setLabel (4,u"Descripción")

	def __conexiones(self):
		self.connect(self.table1,SIGNAL("valueChanged(int,int)"),self.__modificaciontablareg)
		self.connect(self.table2,SIGNAL("valueChanged(int,int)"),self.__modificaciontablavar)

	
	#FUNCIONES SHOW



	def show(self):
		"""Redefinición del show de la clase base"""
		self.__showtablavar()
		self.__showtabladat()
		QTabWidget.show(self)
		

	def __showtablavar(self,pos=None):  
		""" Representa la variable pos en la tabla de variables. Las propiedades de la variable las lee de los datos.
		Si no se indica posicion, se entiende que se quiere rellenar toda la tabla
		"""
		if pos:
			self.__showvar(pos)
			#Actualizacion de titulo en la tabla de datos
			self.__showtitulodat(pos)
		else:
			if self.__nvar>(self.table2.numRows()-self.__interfazdatos.numVar()):
				self.table2.setNumRows(self.__interfazdatos.numVar()+self.__nvar)
			for n in range(self.__interfazdatos.numVar(),self.table2.numRows()):
				for y in range(self.table2.numCols()):
					self.table2.clearCell(n,y)
			for n in range(self.__interfazdatos.numVar()):
				self.__showvar(n)
			self.__showtitulodat()

	def __showvar(self,pos):
		"""Muestra una unica variable (fila) en la tabla segun el listado de variables"""

		from Driza.widgets.widgetsqt import ButtonTableItem
		widget = ButtonTableItem(self.table2,self.__dcasos)

		from Driza.variables import TFactor
		
		variable=self.__interfazdatos.getVar(pos)

		self.table2.setText(pos,0,str(variable.nombre))
		self.table2.setItem(pos,1,self.__combotableitem())	
		self.table2.item(pos,1).setCurrentItem(variable.tipo)
		self.table2.setText(pos,4,str(variable.descripcion))
		self.table2.setItem(pos,2,widget)
		self.table2.setText(pos,3,str(variable.valorpordefecto))

	def __showtabladat(self,pos=None):
		"""Muestra el dato en la posicion pos (fila)
		Lee de los datos, así que deben existir previamente
		Comprueba si esta filtrado o no para cambiar el color
		Si no se indica posicion, se entiende que se quiere rellenar toda la tabla
		"""
		if pos:  
			#Se ha indicado la posición
			self.__showdat(pos)
		else: 
			#No se ha indicado la posicion, se rellena todo
			if self.__nreg>(self.table1.numRows()-self.__interfazdatos.numReg()):
				self.table1.setNumRows(self.__interfazdatos.numReg()+self.__nreg)
			for x in range(self.__interfazdatos.numReg()):
				self.__showdat(x)
			self.__showtitulodat()
			for x in range(self.__interfazdatos.numReg(),self.table1.numRows()):
				for y in range(self.table1.numCols()):
					self.table1.clearCell(x,y)
			for x in range(self.table1.numRows()):
				for y in range(self.__interfazdatos.numVar(),self.table1.numCols()):
					self.table1.clearCell(x,y)

	def __showdat(self,pos):
		"""Muestra un solo dato"""
		from Driza.widgets.widgetsqt import GrisTableItem
		for i in range(self.__interfazdatos.numVar()):
			if self.__gestorfiltro.variable:
				if self.__interfazdatos.getCol(self.__gestorfiltro.variable,filtrado=False)[pos]:
					self.table1.setItem(pos,i,QTableItem(self.table1,QTableItem.OnTyping,str(self.__interfazdatos[pos][i])))
				else:
					self.table1.setItem(pos,i,GrisTableItem(self.table1,QTableItem.OnTyping,str(self.__interfazdatos[pos][i]))) # 
			else:
				self.table1.setText(pos,i,str(self.__interfazdatos[pos][i]))



	def __showtitulodat(self,pos=None):
		"""Actualiza los titulos de la tabla de datos segun las variables"""
		titulos=self.table1.horizontalHeader()
		if pos:
			titulos.setLabel(pos,self.__interfazdatos.getVar(pos).nombre)
		else:
			i=0
			for x in range(self.__interfazdatos.numVar()):
				self.table1.horizontalHeader().setLabel(i,self.__interfazdatos.getVar(i).nombre)
				i+=1

			for x in range(self.__interfazdatos.numVar(),self.table1.horizontalHeader().count()):
				self.table1.horizontalHeader().setLabel(i,'')
				i+=1


	def __combotableitem(self,indice=0):
		"""Devuelve un nuevo objeto tipo combotableitem con la lista de tipos"""
		lista=QStringList()
		from Driza import listas
		for tipo in listas.variables():
			lista.append(tipo)
		return QComboTableItem(self.table2,lista)
	
		
	def __modificaciontablavar(self,x,y):
		"""Funcion a que conecta con la introduccion de nuevos datos en la tabla de variables"""
		self.__portero.guardarestado()
		if x>=self.__interfazdatos.numVar(): #Es una nueva variable
			#Variables intermedias (creadas con los valores por defecto)
			for valorintermedio in range (self.__interfazdatos.numVar(),x):  
				self.__insertarvariable()
				self.__showtablavar(valorintermedio)
			self.__insertarvariable()
		#Variable modificada
		variable=self.__interfazdatos.getVar(x)
		if y==1:   # El usuario quiere cambiar el tipo
			from Driza import variables
			textoencuestion=self.table2.text(x,y).latin1() 
			#preguntar el tipo de conversion
			metododeseado=self.__preguntarconversion(variable,textoencuestion)
			pareja= self.__agenteconversion(variable,textoencuestion,metododeseado) #Pareja variable-list
			self.__interfazdatos.setVar(x,pareja[0],pareja[1])
			self.__showtabladat()  
		elif y==2: # Estamos en el caso de los casos, mejor no hacer nada ... de momento
			print "No implementado"
		else: #Restos de campos (Texto)
			from Driza.excepciones import VariableExisteException
			try:
				self.__interfazdatos.modVar(variable,y,str(self.table2.text(x,y).latin1()))
			except VariableExisteException:
				rc = QMessageBox.warning(self,'Atención', u'El nombre de variable ya existe')


		self.__showtablavar()  
		#En todos los casos, actualizamos el titulo de la tabla de datos
		self.__showtitulodat()

	def __actualizarreginterfazdatos(self,x,y,valor):
		"""actualiza un dato en interfazdatos,recogiendo la excepcion en caso de error """
		try:
			self.__interfazdatos[x][y]=self.__interfazdatos.getVar(y).nuevoitem(valor)
		except ValueError:
			rc = QMessageBox.warning(self,'Atención', u'El dato no es válido')
	
	def __insertarregistro(self):
		"""Inserta un registro genérico, y ademas comprueba que no nos estamos acercando al final de la tabla"""
		self.__interfazdatos.appReg()
		if self.__nreg>(self.table1.numRows()-self.__interfazdatos.numReg()):
			self.table1.setNumRows(self.table1.numRows()+1)

	def __insertarvariable(self):
		"""Inserta una variable genérica, y ademas comprueba que no nos acercamos a ninguno de los limites de las tablas"""
		self.__interfazdatos.appVar()
		if self.__nvar>(self.table2.numRows()-self.__interfazdatos.numVar()):
			self.table2.setNumRows(self.table2.numRows()+1)
			self.table1.setNumCols(self.table1.numCols()+1)

	def __modificaciontablareg(self,x,y):
		"""Actualiza los datos del objeto dato a partir de un cambio en la tabla de datos"""
		self.__portero.guardarestado()
		#from Driza import datos
		valor=self.table1.text(x,y).latin1()
		if y>=self.__interfazdatos.numVar():
			#Estamos trabajando sobre una variable inexistente
			for i in range(self.__interfazdatos.numVar(),y):
				self.__insertarvariable()
			self.__interfazdatos.appVar()
			self.__showtablavar() #actualizamos la tabla de variables
		if x>=self.__interfazdatos.numReg():
			#no existen registros intermedios
			for i in range (self.__interfazdatos.numReg(),x):  
				self.__insertarregistro()
				self.__showtabladat(i)
			self.__interfazdatos.appReg()
		#Actualizamos la fila que ha utilizado el usuario
		self.__actualizarreginterfazdatos(x,y,valor)
		self.__showtabladat()
		self.parent().parent().parent().showundoredo()

	
	def removeselection(self,borrardatos=False):
		"""Esta funcion borra la seleccion de la tabla. El parametro opcional determina si se ha deborrar tambien los datos"""
		tablaactual=self.currentPageIndex()
		assert(tablaactual == 0) #Cambiar por una excepcion más concreta
		tabla=self.table1
		if borrardatos:
			lista=[]
			for seleccion in range(tabla.numSelections()):
				for fila in range(tabla.selection(seleccion).topRow(),tabla.selection(seleccion).bottomRow()+1):
					lista.append(fila)
			lista.sort()
			lista.reverse()
			for fila in lista:
				del self.__interfazdatos[fila]

		for seleccion in range(tabla.numSelections()):
			tabla.removeSelection(seleccion)

	def __preguntarconversion(self,variable,objetivo):
		"""Pregunta cual de los metodos disponibles para la conversion desea escoger el usuario"""
		#Leer el diccionario de variable con la key objetivo
		tipoinicial="T"+objetivo
		lista=[]
		for tipo in variable.diccionarioconversion.iterkeys():
			if issubclass(eval(tipoinicial),eval(tipo)):
				lista+=variable.diccionarioconversion[tipo]
		#Elaborar una lista y preguntar al usuario
		qlista=QStringList()
		for elemento in lista:
			qlista.append(elemento)
		cadena=QInputDialog.getItem("Elige!",u"Elige una función de conversion",qlista,0,False,self,"Dialogo")
		#devolver el nombre de la funcion
		return cadena[0].latin1()

	def listaseleccion(self,tabla=0):
		"""Devuelve una lista con los registros que han sido seleccionados"""
		tabla=self.table1
		cadena=""
		lista=[]
		listasalida=[]
		for seleccion in range(tabla.numSelections()):
			for fila in range(tabla.selection(seleccion).topRow(),tabla.selection(seleccion).bottomRow()+1):
				lista.append(fila)
		lista.sort() #En la lista estan todas las filas que se nos piden y ordenadas
		for x in lista:
			listasalida.append(self.__interfazdatos[x])

		return listasalida

