#!/usr/bin/python
# -*- coding: utf-8 -*-

#Clases que gestionan la selección

#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from qt import *
from Driza.ui.widgets import uiselectorsimple
from Driza.ui.widgets import uiselectordoble
from Driza.clasesbase import ElementoWidgetOperacion

class Selector(ElementoWidgetOperacion):
	"""Clase padre de los selectores"""
	def __init__(self,padre,dato):
		#VARIABLES PUBLICAS
		self.seleccion=[]
		self.parent=padre
		self.dato=dato



class Selectorsimple(Selector,uiselectorsimple.WSelectorSimple):
	"""Este selector posee dos cajas y dos botones, que son pasados en la inicialización. Una de las cajas almacena todas las posibilidades,mientras que la otra almacena la seleccion"""
	def __init__(self,padre,dato):
		"""Clase de inicialización, almacena los componentes"""
		Selector.__init__(self,padre,dato)
		uiselectorsimple.WSelectorSimple.__init__(self,padre)
		self.__conexiones()

	#FUNCIONES PUBLICAS
	
	def show(self):
		"""
		Actualiza todos los valores del selector
		Es llamada por el show del dialogo al que pertenece
		"""
		self.cajadisponible.clear()
		self.cajaseleccion.clear()
		for var in self.dato.arrTit(factores=False):
			if not var in self.seleccion:
				self.cajadisponible.insertItem(var)
		for var in self.seleccion:
			self.cajaseleccion.insertItem(var)
		uiselectorsimple.WSelectorSimple.show(self)
	
	#FUNCIONES PRIVADAS

	def __conexiones(self):
		"""Establece las conexiones entre elementos"""
		self.connect(self.pushButton1,SIGNAL("clicked()"),self.__addelemento)
		self.connect(self.pushButton2,SIGNAL("clicked()"),self.__delelemento)

	def __addelemento(self):
		"""Añade un elemento de la seleccion de la cajadisponible"""
		if self.cajadisponible.currentText():
			self.seleccion.append(self.cajadisponible.currentText())
			self.show()
	
	def __delelemento(self):
		"""Borra el elemento seleccionado en lacaja de seleccion"""
		if self.cajaseleccion.currentText():
			self.seleccion.remove(self.cajaseleccion.currentText())
			self.show()

class Selectordoble(Selector,uiselectordoble.WSelectorDoble):
	"""Este selector posee tres cajas y dos botones, que son pasados en la inicialización. Una de las cajas almacena todas las posibilidades,mientras que la otra almacena la seleccion"""

	def __init__(self,padre,dato):
		"""Clase de inicialización, almacena los componentes"""
		Selector.__init__(self,padre,dato)
		uiselectordoble.WSelectorDoble.__init__(self,padre)

		self.__conexiones()

		#VARIABLES PRIVADAS

	#FUNCIONES PUBLICAS

	def show(self):
		"""
		Actualiza todos los valores del selector
		Es llamada por el show del dialogo al que pertenece
		"""
		self.cajadisponible1.clear()
		self.cajadisponible2.clear()
		self.cajaseleccion.clear()
		for var in self.dato.arrTit(factores=False):
			self.cajadisponible1.insertItem(var)
			self.cajadisponible2.insertItem(var)
		for var in self.seleccion:
			self.cajaseleccion.insertItem(var[0].latin1()+"-"+var[1].latin1())
		uiselectordoble.WSelectorDoble.show(self)

	#FUNCIONES PRIVADAS
	
	def __conexiones(self):
		"""Bloque de conexiones"""
		self.connect(self.pushButton1,SIGNAL("clicked()"),self.__addelemento)
		self.connect(self.pushButton2,SIGNAL("clicked()"),self.__delelemento)

	def __addelemento(self):
		"""Añade un elemento de la seleccion de la cajadisponible"""
		if self.cajadisponible1.currentText() and self.cajadisponible2.currentText():
			self.seleccion.append([self.cajadisponible1.currentText(),self.cajadisponible2.currentText()])
			self.show()
	
	def __delelemento(self):
		"""Borra una pareja de la caja de seleccion"""
		if self.__cajaseleccion.currentItem():
			del self.seleccion[self.__cajaseleccion.currentItem()]
			self.show()


class SelectorVariableValor(Selector,uiselectordoble.WSelectorDoble):
	"""Permite escoger varias parejas variable-valor en vez de solamente variables. Utilizadas en las proporciones"""
	def __init__(self,padre,dato):
		"""Clase de inicialización, almacena los componentes"""
		Selector.__init__(self,padre,dato)
		uiselectordoble.WSelectorDoble.__init__(self,padre)
		self.textLabel2.setText("<b>Caso de la variable</b>")

		self.__conexiones()

		#VARIABLES PRIVADAS

	#FUNCIONES PUBLICAS

	def show(self):
		"""
		Actualiza todos los valores del selector
		Es llamada por el show del dialogo al que pertenece
		"""
		self.cajadisponible1.clear()
		self.cajadisponible2.clear()
		self.cajaseleccion.clear()
		for var in self.dato.arrTit(factores=True):
			self.cajadisponible1.insertItem(var)
		self.__showcaja2()
		for var in self.seleccion:
			self.cajaseleccion.insertItem(var[0].latin1()+"-"+var[1].latin1())
		uiselectordoble.WSelectorDoble.show(self)

	#FUNCIONES PRIVADAS
	
	def __conexiones(self):
		"""Bloque de conexiones"""
		self.connect(self.pushButton1,SIGNAL("clicked()"),self.__addelemento)
		self.connect(self.pushButton2,SIGNAL("clicked()"),self.__delelemento)
		self.connect(self.cajadisponible1,SIGNAL("selectionChanged()"),self.__showcaja2)

	def __addelemento(self):
		"""Añade un elemento de la seleccion de la cajadisponible"""
		if self.cajadisponible1.currentText() and self.cajadisponible2.currentText():
			self.seleccion.append([self.cajadisponible1.currentText(),self.cajadisponible2.currentText()])
			self.show()
	
	def __delelemento(self):
		"""Borra una pareja de la caja de seleccion"""
		if self.__cajaseleccion.currentItem():
			del self.seleccion[self.__cajaseleccion.currentItem()]
			self.show()

	def __showcaja2(self):
		"""Muestra los casos de la variable de la caja1 en la caja2"""
		self.cajadisponible2.clear()
		listacasos=self.dato.getCasos(self.cajadisponible1.currentText().latin1())
		for caso in listacasos:
			self.cajadisponible2.insertItem(str(caso))

