#!/usr/bin/python
# -*- coding: utf-8 -*-


#Copyright (C) 2006  Inmaculada Luengo Merino, Nstor Arocha Rodrguez
#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#Fichero de libreria de qt
from qt import *
from qttable import *


#http://www.digitalfanatics.org/projects/qt_tutorial/chapter13.html
#Mirar de http://www.handhelds.org/~zecke/apidocs/libopie/html/ocheckitem_8cpp-source.html
#Mirar tablestatistics.py de ejemplos de PyQT

class ButtonTableItem(QTableItem):
	"""Esta clase hereda de QTableItem. Modifica dos aspectos de estos:
		Por un lado inhabilita el texto que muestra normalmente
		Por otro, utiliza un botón para que al ser pulsado lanza el dialogo de casos
		"""

	def __init__(self,tabla,dcasos):
		"""la inicialización llama al padre"""
		QTableItem.__init__(self,tabla,QTableItem.WhenCurrent,"...") 
		self.setReplaceable(False)
		self.__dcasos=dcasos

	#FUNCIONES PUBLICAS
		
	def createEditor(self):
		"""Crea un editor para interactuar con el contenido.
		En este caso es un botón, ya que permite asignar una acción al click de ratón"""
		boton=QButton(self.table().viewport(),"Ventana abierta") #El padre es viewport, es invisible
		QObject.connect(boton,SIGNAL("clicked()"),self.__editar)
		QObject.connect(boton,SIGNAL("clicked()"),self.table(), SLOT("doValueChanged()"))
		return boton

	def setContentFromEditor(self,widget):
		"""Cuando el contenido de widget cambia, qt llama a esta función.
		Esta desactivada ya que el botón que actua de editor no dispone de información para devolver"""
		pass

	#FUNCIONES PRIVADAS

	def __editar(self):
		"""Abre el dialogo de casos, pasando la columna como valor"""
		self.__dcasos.show(self.row())

#TableItem modificado para mostrar el fondo gris
#http://mats.imk.fraunhofer.de/pipermail/pykde/2003-October/006268.html

class GrisTableItem(QTableItem):
	"""Widget que es equivalente en funcionalidad a un QTableItem, solamente cambia el color del fondo en la función de dibujado"""

	#FUNCIONES PUBLICAS

	def paint(self, painter, colorgroup, rect, selected):
		"""Redefine la clase padre poniendo el fondo gris"""
		cg = QColorGroup(colorgroup)
		cg.setColor(QColorGroup.Base, QColor("Gray"))
		QTableItem.paint(self,painter, cg, rect, selected)



class ComboBoxVars(QComboBox):
	"""ComboBox modificado que devuelve siempre una lista de variables actualizadas"""
	def __init__(self,parent,datos,factores=True):
		self.__datos=datos
		QComboBox.__init__(self,parent,"comboboxvars")
		self.__factores=factores
		
	def show(self):
		"""Redefine el show de QComboBox, añadiendo el listado de variables"""
		#texto=self.currentText()
		self.clear()
		for variable in self.__datos.arrTit(factores=self.__factores):
			self.insertItem(variable)
		#self.setCurrentText(texto) #TODO: dejar marcado el item que estaba
		QComboBox.show(self)

class ComboBoxFact(QComboBox):
	"""ComboBox modificado que devuelve siempre una lista de variables actualizadas"""
	def __init__(self,parent,datos):
		self.__datos=datos
		QComboBox.__init__(self,parent,"comboboxfact")
		
	def show(self):
		"""Redefine el show de QComboBox, añadiendo el listado de variables"""
		#texto=self.currentText()
		self.clear()
		for variable in self.__datos.arrTit(tipo="Factor"):
			self.insertItem(variable)
		#self.setCurrentText(texto) #TODO: dejar marcado el item que estaba
		QComboBox.show(self)
