#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#woperaciones.py: Todos los widgets de operaciones

#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from qt import *
from Driza.widgets import seleccion


class WidgetOperacion(QWidget):
	"""Clase base de todos los widgets de operaciones"""
	def __init__(self,parent,name):
		if isinstance(name,unicode):
			name=name.encode("latin1")
		if not name:
			self.setName("WidgetOperacion")
		QWidget.__init__(self,parent,name,0)

		self.rejilla=QGridLayout(self,1,1,11,6,"rejilla")
		self.languageChange()
		self.resize(QSize(600,480).expandedTo(self.minimumSizeHint()))
		self.clearWState(Qt.WState_Polished)


	def languageChange(self):
		self.setCaption(self.__tr("Form1"))

	def __tr(self,s,c = None):
		return qApp.translate("Form1",s,c)
	
	def seleccion(self):
		pass

	def opciones(self):
		return self.wopciones.opciones()


class WidgetCalculoSimple(WidgetOperacion):
	"""Es un widget de calculo, con un selector simple y un widget de opciones suministrado en el constructor"""
	def __init__(self,name,listaopciones,datos):
		from Driza.widgets.wopciones import WOpciones
		WidgetOperacion.__init__(self,None,name)
		self.cajas=seleccion.Selectorsimple(self,datos)
		self.rejilla.addWidget(self.cajas,0,0)
		self.wopciones=WOpciones(datos)
		for x in listaopciones:
			self.wopciones.parse(x)
		self.zonastack=QWidgetStack(self,"stack")
		self.rejilla.addWidget(self.zonastack,1,0)
		self.zonastack.setMinimumSize(QSize(200,200))
		self.zonastack.addWidget(self.wopciones)
		self.zonastack.raiseWidget(self.wopciones)

	def seleccion(self):
		return self.cajas.seleccion

class WidgetCalculoDoble(WidgetOperacion):
	"""Es un widget de calculo, con un selector doble y un widget de opciones suministrado en el constructor"""
	def __init__(self,name,listaopciones,datos):
		from Driza.widgets.wopciones import WOpciones
		WidgetOperacion.__init__(self,None,name)
		self.cajas=seleccion.Selectordoble(self,datos)
		self.rejilla.addWidget(self.cajas,0,0)
		self.wopciones=WOpciones(datos)
		for x in listaopciones:
			self.wopciones.parse(x)
		self.zonastack=QWidgetStack(self,"stack")
		self.rejilla.addWidget(self.zonastack,1,0)
		self.zonastack.setMinimumSize(QSize(200,200))
		self.zonastack.addWidget(self.wopciones)
		self.zonastack.raiseWidget(self.wopciones)

	def seleccion(self):
		return self.cajas.seleccion


class WidgetCalculoSimpleBondadAjuste(WidgetCalculoSimple):
	"""Es un widget especifico para todas las operaciones de bondad de ajuste"""
	def __init__(self,name,datos):
#		from Driza.widgets.wopciones import WOpciones
		listawidget=[]
		listawidget.append({"tipo":"SeleccionSimple","nombre":u"Distribución","opciones":["Uniforme"]})
		listawidget.append({"tipo":"ListaSimple","nombre":u"Percentil","opciones":["95","90"]})
		WidgetCalculoSimple.__init__(self,name,listawidget,datos)
		
class WidgetGraficos(WidgetOperacion):
	"""Widget de gráficos"""
	def __init__(self,name,listagraficos,listaopciones,datos):
		from Driza.widgets.wopciones import WOpciones
		WidgetOperacion.__init__(self,None,name)
		widget=QLabel(self)
		widget.setText(u"Prueba de gráficos")
		self.rejilla.addWidget(widget,0,0)
		self.wgraficos=WOpciones(datos)
		self.wopciones=WOpciones(datos)
		for x in listaopciones:
			self.wopciones.parse(x)
		for x in listagraficos:
			self.wgraficos.parse(x)
		self.zonastack1=QWidgetStack(self,"stack")
		self.rejilla.addWidget(self.zonastack1,1,0)
		self.zonastack1.setMinimumSize(QSize(200,200))
		self.zonastack1.addWidget(self.wgraficos)
		self.zonastack1.raiseWidget(self.wgraficos)
		self.zonastack2=QWidgetStack(self,"stack")
		self.rejilla.addWidget(self.zonastack2,2,0)
		self.zonastack2.setMinimumSize(QSize(200,200))
		self.zonastack2.addWidget(self.wopciones)
		self.zonastack2.raiseWidget(self.wopciones)
		
	def ejes(self):
		return self.wgraficos.opciones()


class WidgetCaso(WidgetOperacion):
	"""Es un widget de calculo, con un selector variablevalor  y un widget de opciones suministrado en el constructor"""
	def __init__(self,name,listaopciones,datos):
		from Driza.widgets.wopciones import WOpciones
		WidgetOperacion.__init__(self,None,name)
		self.cajas=seleccion.SelectorVariableValor(self,datos)
		self.rejilla.addWidget(self.cajas,0,0)
		self.wopciones=WOpciones(datos)
		for x in listaopciones:
			self.wopciones.parse(x)
		self.zonastack=QWidgetStack(self,"stack")
		self.rejilla.addWidget(self.zonastack,1,0)
		self.zonastack.setMinimumSize(QSize(200,200))
		self.zonastack.addWidget(self.wopciones)
		self.zonastack.raiseWidget(self.wopciones)

	def seleccion(self):
		return self.cajas.seleccion

class WidgetDistribucion(WidgetOperacion):
	"""Widget idoneo para los calculos asociados a las distribuciones"""
	def __init__(self,name,listacuantilesprob,listagrafica,datos):
		WidgetOperacion.__init__(self,None,name)
		from Driza.widgets.wopciones import WOpciones
		widget=WOpciones(datos)
		self.wopciones=widget
		widget.addCO("Cuantiles")
		widget.addCO("Probabilidades")
		widget.addCO(u"Gráfica")
		widget.lista[0].addETS("Probabilidad")
		widget.lista[1].addETS("Valores")
		for x in listacuantilesprob:
			widget.lista[0].parse(x)
			widget.lista[1].parse(x)
			widget.lista[2].parse(x)
		widget.lista[0].parse({"nombre":u"DirecciónCola","tipo":"SeleccionSimple","opciones":["izquierda","derecha"]})
		widget.lista[1].parse({"nombre":u"DirecciónCola","tipo":"SeleccionSimple","opciones":["izquierda","derecha"]})
		widget.lista[2].parse({"tipo":"SeleccionSimple","nombre":"Tipografica","opciones":["Densidad",u"Distribución"]})
		widget.lista[0].setFrameStyle(QFrame.Sunken)
		widget.lista[0].setTitle("")
		widget.lista[1].setFrameStyle(QFrame.Sunken)
		widget.lista[1].setTitle("")
		widget.lista[2].setFrameStyle(QFrame.Sunken)
		widget.lista[2].setTitle("")
		self.zonastack=QWidgetStack(self,"stack")
		self.rejilla.addWidget(self.zonastack,0,0)
		self.zonastack.setMinimumSize(QSize(200,200))
		self.zonastack.addWidget(widget)
		self.zonastack.raiseWidget(widget)
		self.wopciones=widget
		
